@@ -8,7 +8,9 @@ from typing import List, Tuple, Dict, Optional, Union
 import numpy as np
 from miditoolkit import Instrument, Note, TempoChange
 
-from .midi_tokenizer_base import MIDITokenizer, Vocabulary, Event, detect_chords
+from .midi_tokenizer_base import MIDITokenizer
+from .vocabulary import Vocabulary, Event
+from .utils import detect_chords
 from .constants import *
 
 
